@@ -1,13 +1,12 @@
 use core::fmt::{self, Debug, Formatter};
 use core::intrinsics::assume;
-use core::iter::{FusedIterator, TrustedLen, TrustedRandomAccess};
+use core::iter::{FusedIterator, TrustedLen, TrustedRandomAccessNoCoerce};
 use core::marker::{PhantomData, Send, Sync};
 use core::mem::{replace, size_of, MaybeUninit};
 use core::ptr;
+use core::slice::{from_raw_parts, from_raw_parts_mut};
 
-use crate::utils::{
-  distance_between, slice_from_raw_parts, slice_from_raw_parts_mut, zst_ptr_add, zst_ptr_add_mut,
-};
+use crate::utils::{distance_between, zst_ptr_add, zst_ptr_add_mut};
 use crate::StaticVec;
 
 #[cfg(feature = "std")]
@@ -93,7 +92,7 @@ impl<'a, T: 'a, const N: usize> StaticVecIterConst<'a, T, N> {
   #[inline(always)]
   pub const fn as_slice(&self) -> &'a [T] {
     // Safety: `start` is never null. This function will "at worst" return an empty slice.
-    slice_from_raw_parts(self.start, distance_between(self.end, self.start))
+    unsafe { from_raw_parts(self.start, distance_between(self.end, self.start)) }
   }
 }
 
@@ -101,7 +100,7 @@ impl<'a, T: 'a, const N: usize> StaticVecIterConst<'a, T, N> {
 // future where `for` loops are allowed in `const` contexts, and do not necessarily benefit in any
 // particularly useful way from being `const` impls quite yet.
 
-impl<'a, T: 'a, const N: usize> const Iterator for StaticVecIterConst<'a, T, N> {
+impl<'a, T: 'a, const N: usize> Iterator for StaticVecIterConst<'a, T, N> {
   type Item = &'a T;
 
   #[inline(always)]
@@ -177,7 +176,7 @@ impl<'a, T: 'a, const N: usize> const Iterator for StaticVecIterConst<'a, T, N>
   }
 }
 
-impl<'a, T: 'a, const N: usize> const DoubleEndedIterator for StaticVecIterConst<'a, T, N> {
+impl<'a, T: 'a, const N: usize> DoubleEndedIterator for StaticVecIterConst<'a, T, N> {
   #[inline(always)]
   fn next_back(&mut self) -> Option<&'a T> {
     unsafe {
@@ -234,13 +233,18 @@ unsafe impl<'a, T: 'a, const N: usize> const TrustedLen for StaticVecIterConst<'
 // We hide this one just in case it gets removed from `std` later, so no one relies on us relying on
 // it.
 #[doc(hidden)]
-unsafe impl<'a, T: 'a, const N: usize> const TrustedRandomAccess for StaticVecIterConst<'a, T, N> {
+unsafe impl<'a, T: 'a, const N: usize> const TrustedRandomAccessNoCoerce
+  for StaticVecIterConst<'a, T, N>
+{
   const MAY_HAVE_SIDE_EFFECT: bool = false;
+  fn size(&self) -> usize {
+    distance_between(self.end, self.start)
+  }
 }
 unsafe impl<'a, T: 'a + Sync, const N: usize> const Sync for StaticVecIterConst<'a, T, N> {}
 unsafe impl<'a, T: 'a + Sync, const N: usize> const Send for StaticVecIterConst<'a, T, N> {}
 
-impl<'a, T: 'a, const N: usize> const Clone for StaticVecIterConst<'a, T, N> {
+impl<'a, T: 'a, const N: usize> Clone for StaticVecIterConst<'a, T, N> {
   #[inline(always)]
   fn clone(&self) -> Self {
     Self {
@@ -289,11 +293,11 @@ impl<'a, T: 'a, const N: usize> StaticVecIterMut<'a, T, N> {
   #[inline(always)]
   pub const fn as_slice(&self) -> &[T] {
     // Safety: `start` is never null. This function will "at worst" return an empty slice.
-    slice_from_raw_parts(self.start, distance_between(self.end, self.start))
+    unsafe { from_raw_parts(self.start, distance_between(self.end, self.start)) }
   }
 }
 
-impl<'a, T: 'a, const N: usize> const Iterator for StaticVecIterMut<'a, T, N> {
+impl<'a, T: 'a, const N: usize> Iterator for StaticVecIterMut<'a, T, N> {
   type Item = &'a mut T;
 
   #[inline(always)]
@@ -369,7 +373,7 @@ impl<'a, T: 'a, const N: usize> const Iterator for StaticVecIterMut<'a, T, N> {
   }
 }
 
-impl<'a, T: 'a, const N: usize> const DoubleEndedIterator for StaticVecIterMut<'a, T, N> {
+impl<'a, T: 'a, const N: usize> DoubleEndedIterator for StaticVecIterMut<'a, T, N> {
   #[inline(always)]
   fn next_back(&mut self) -> Option<&'a mut T> {
     unsafe {
@@ -426,8 +430,13 @@ unsafe impl<'a, T: 'a, const N: usize> const TrustedLen for StaticVecIterMut<'a,
 // We hide this one just in case it gets removed from `std` later, so no one relies on us relying on
 // it.
 #[doc(hidden)]
-unsafe impl<'a, T: 'a, const N: usize> const TrustedRandomAccess for StaticVecIterMut<'a, T, N> {
+unsafe impl<'a, T: 'a, const N: usize> const TrustedRandomAccessNoCoerce
+  for StaticVecIterMut<'a, T, N>
+{
   const MAY_HAVE_SIDE_EFFECT: bool = false;
+  fn size(&self) -> usize {
+    distance_between(self.end, self.start)
+  }
 }
 unsafe impl<'a, T: 'a + Sync, const N: usize> const Sync for StaticVecIterMut<'a, T, N> {}
 unsafe impl<'a, T: 'a + Send, const N: usize> const Send for StaticVecIterMut<'a, T, N> {}
@@ -475,7 +484,7 @@ impl<T, const N: usize> StaticVecIntoIter<T, N> {
         _ => StaticVec::first_ptr(&self.data).add(self.start),
       }
     };
-    slice_from_raw_parts(start_at, self.end - self.start)
+    unsafe { from_raw_parts(start_at, self.end - self.start) }
   }
 
   /// Returns a mutable slice consisting of the elements in the range between the iterator's
@@ -489,7 +498,7 @@ impl<T, const N: usize> StaticVecIntoIter<T, N> {
         _ => StaticVec::first_ptr_mut(&mut self.data).add(self.start),
       }
     };
-    slice_from_raw_parts_mut(start_at, self.end - self.start)
+    unsafe { from_raw_parts_mut(start_at, self.end - self.start) }
   }
 }
 
@@ -542,7 +551,7 @@ impl<T, const N: usize> Iterator for StaticVecIntoIter<T, N> {
           0 => zst_ptr_add_mut(StaticVec::first_ptr_mut(&mut self.data), old_start),
           _ => StaticVec::first_ptr_mut(&mut self.data).add(old_start),
         };
-        ptr::drop_in_place(slice_from_raw_parts_mut(drop_at, res_index - old_start));
+        ptr::drop_in_place(from_raw_parts_mut(drop_at, res_index - old_start));
         // Adjust our starting index.
         self.start = res_index + 1;
         Some(res.read())
@@ -557,9 +566,9 @@ impl<T, const N: usize> Iterator for StaticVecIntoIter<T, N> {
 
   #[inline(always)]
   unsafe fn __iterator_get_unchecked(&mut self, idx: usize) -> T
-  where Self: TrustedRandomAccess {
-    // Safety: `TrustedRandomAccess` is only implemented for `StaticVecIntoIter` when `T` is `Copy`,
-    // and the caller is required to uphold the invariants for this function.
+  where Self: TrustedRandomAccessNoCoerce {
+    // Safety: `TrustedRandomAccessNoCoerce` is only implemented for `StaticVecIntoIter` when `T` is
+    // `Copy`, and the caller is required to uphold the invariants for this function.
     match size_of::<T>() {
       0 => zst_ptr_add(StaticVec::first_ptr(&self.data), idx).read(),
       _ => StaticVec::first_ptr(&self.data).add(idx).read(),
@@ -599,7 +608,7 @@ impl<T, const N: usize> DoubleEndedIterator for StaticVecIntoIter<T, N> {
           0 => zst_ptr_add_mut(StaticVec::first_ptr_mut(&mut self.data), res_index),
           _ => StaticVec::first_ptr_mut(&mut self.data).add(res_index),
         };
-        ptr::drop_in_place(slice_from_raw_parts_mut(drop_at, old_end - res_index));
+        ptr::drop_in_place(from_raw_parts_mut(drop_at, old_end - res_index));
         match size_of::<T>() {
           0 => Some(zst_ptr_add(StaticVec::first_ptr(&self.data), self.end).read()),
           _ => Some(StaticVec::first_ptr(&self.data).add(self.end).read()),
@@ -609,7 +618,7 @@ impl<T, const N: usize> DoubleEndedIterator for StaticVecIntoIter<T, N> {
   }
 }
 
-impl<T, const N: usize> ExactSizeIterator for StaticVecIntoIter<T, N> {
+impl<T, const N: usize> const ExactSizeIterator for StaticVecIntoIter<T, N> {
   #[inline(always)]
   fn len(&self) -> usize {
     self.end - self.start
@@ -621,16 +630,20 @@ impl<T, const N: usize> ExactSizeIterator for StaticVecIntoIter<T, N> {
   }
 }
 
-impl<T, const N: usize> FusedIterator for StaticVecIntoIter<T, N> {}
-unsafe impl<T, const N: usize> TrustedLen for StaticVecIntoIter<T, N> {}
+impl<T, const N: usize> const FusedIterator for StaticVecIntoIter<T, N> {}
+unsafe impl<T, const N: usize> const TrustedLen for StaticVecIntoIter<T, N> {}
 // We hide this one just in case it gets removed from `std` later, so no one relies on us relying on
 // it.
 #[doc(hidden)]
-unsafe impl<T: Copy, const N: usize> const TrustedRandomAccess for StaticVecIntoIter<T, N> {
+unsafe impl<T: Copy, const N: usize> const TrustedRandomAccessNoCoerce for StaticVecIntoIter<T, N> {
   const MAY_HAVE_SIDE_EFFECT: bool = false;
+
+  fn size(&self) -> usize {
+    self.end - self.start
+  }
 }
-unsafe impl<T: Sync, const N: usize> Sync for StaticVecIntoIter<T, N> {}
-unsafe impl<T: Send, const N: usize> Send for StaticVecIntoIter<T, N> {}
+unsafe impl<T: Sync, const N: usize> const Sync for StaticVecIntoIter<T, N> {}
+unsafe impl<T: Send, const N: usize> const Send for StaticVecIntoIter<T, N> {}
 
 impl<T: Clone, const N: usize> Clone for StaticVecIntoIter<T, N> {
   #[inline(always)]
@@ -662,7 +675,7 @@ impl<T, const N: usize> Drop for StaticVecIntoIter<T, N> {
     };
     match item_count {
       0 => (),
-      _ => unsafe { ptr::drop_in_place(slice_from_raw_parts_mut(drop_at, item_count)) },
+      _ => unsafe { ptr::drop_in_place(from_raw_parts_mut(drop_at, item_count)) },
     }
   }
 }
@@ -716,9 +729,9 @@ impl<'a, T: 'a, const N: usize> Iterator for StaticVecDrain<'a, T, N> {
 
   #[inline(always)]
   unsafe fn __iterator_get_unchecked(&mut self, idx: usize) -> T
-  where Self: TrustedRandomAccess {
-    // Safety: `TrustedRandomAccess` is only implemented for `StaticVecDrain` when `T` is `Copy`,
-    // and the caller is required to uphold the invariants for this function.
+  where Self: TrustedRandomAccessNoCoerce {
+    // Safety: `TrustedRandomAccessNoCoerce` is only implemented for `StaticVecDrain` when `T` is
+    // `Copy`, and the caller is required to uphold the invariants for this function.
     match size_of::<T>() {
       0 => zst_ptr_add(self.as_slice().as_ptr(), idx).read(),
       _ => self.as_slice().as_ptr().add(idx).read(),
@@ -736,7 +749,7 @@ impl<'a, T: 'a, const N: usize> DoubleEndedIterator for StaticVecDrain<'a, T, N>
   }
 }
 
-impl<'a, T: 'a, const N: usize> ExactSizeIterator for StaticVecDrain<'a, T, N> {
+impl<'a, T: 'a, const N: usize> const ExactSizeIterator for StaticVecDrain<'a, T, N> {
   #[inline(always)]
   fn len(&self) -> usize {
     self.iter.len()
@@ -748,18 +761,21 @@ impl<'a, T: 'a, const N: usize> ExactSizeIterator for StaticVecDrain<'a, T, N> {
   }
 }
 
-impl<'a, T: 'a, const N: usize> FusedIterator for StaticVecDrain<'a, T, N> {}
-unsafe impl<'a, T: 'a, const N: usize> TrustedLen for StaticVecDrain<'a, T, N> {}
+impl<'a, T: 'a, const N: usize> const FusedIterator for StaticVecDrain<'a, T, N> {}
+unsafe impl<'a, T: 'a, const N: usize> const TrustedLen for StaticVecDrain<'a, T, N> {}
 // We hide this one just in case it gets removed from `std` later, so no one relies on us relying on
 // it.
 #[doc(hidden)]
-unsafe impl<'a, T: Copy + 'a, const N: usize> const TrustedRandomAccess
+unsafe impl<'a, T: Copy + 'a, const N: usize> const TrustedRandomAccessNoCoerce
   for StaticVecDrain<'a, T, N>
 {
   const MAY_HAVE_SIDE_EFFECT: bool = false;
+  fn size(&self) -> usize {
+    distance_between(self.iter.end, self.iter.start)
+  }
 }
-unsafe impl<'a, T: 'a + Sync, const N: usize> Sync for StaticVecDrain<'a, T, N> {}
-unsafe impl<'a, T: 'a + Send, const N: usize> Send for StaticVecDrain<'a, T, N> {}
+unsafe impl<'a, T: 'a + Sync, const N: usize> const Sync for StaticVecDrain<'a, T, N> {}
+unsafe impl<'a, T: 'a + Send, const N: usize> const Send for StaticVecDrain<'a, T, N> {}
 
 impl<'a, T: 'a + Debug, const N: usize> Debug for StaticVecDrain<'a, T, N> {
   #[inline(always)]
@@ -855,7 +871,9 @@ impl<T, I: Iterator<Item = T> + DoubleEndedIterator, const N: usize> DoubleEnded
   }
 }
 
-impl<T, I: Iterator<Item = T>, const N: usize> ExactSizeIterator for StaticVecSplice<T, I, N> {
+impl<T, I: Iterator<Item = T>, const N: usize> const ExactSizeIterator
+  for StaticVecSplice<T, I, N>
+{
   #[inline(always)]
   fn len(&self) -> usize {
     self.end - self.start
@@ -867,10 +885,19 @@ impl<T, I: Iterator<Item = T>, const N: usize> ExactSizeIterator for StaticVecSp
   }
 }
 
-impl<T, I: Iterator<Item = T>, const N: usize> FusedIterator for StaticVecSplice<T, I, N> {}
-unsafe impl<T, I: Iterator<Item = T>, const N: usize> TrustedLen for StaticVecSplice<T, I, N> {}
-unsafe impl<T: Sync, I: Iterator<Item = T>, const N: usize> Sync for StaticVecSplice<T, I, N> {}
-unsafe impl<T: Send, I: Iterator<Item = T>, const N: usize> Send for StaticVecSplice<T, I, N> {}
+impl<T, I: Iterator<Item = T>, const N: usize> const FusedIterator for StaticVecSplice<T, I, N> {}
+unsafe impl<T, I: Iterator<Item = T>, const N: usize> const TrustedLen
+  for StaticVecSplice<T, I, N>
+{
+}
+unsafe impl<T: Sync, I: Iterator<Item = T>, const N: usize> const Sync
+  for StaticVecSplice<T, I, N>
+{
+}
+unsafe impl<T: Send, I: Iterator<Item = T>, const N: usize> const Send
+  for StaticVecSplice<T, I, N>
+{
+}
 
 impl<T: Debug, I: Iterator<Item = T>, const N: usize> Debug for StaticVecSplice<T, I, N> {
   #[inline(always)]
@@ -878,7 +905,7 @@ impl<T: Debug, I: Iterator<Item = T>, const N: usize> Debug for StaticVecSplice<
     // Safety: `self.vec` will never be null, and `self.end` and `self.start` will always be within
     // an appropriate range.
     unsafe {
-      let items = slice_from_raw_parts(
+      let items = from_raw_parts(
         (&*self.vec).ptr_at_unchecked(self.start),
         self.end - self.start,
       );
