@@ -29,26 +29,21 @@
   incomplete_features
 )]
 #![feature(
+  adt_const_params,
   const_assume,
-  const_evaluatable_checked,
   const_fn_floating_point_arithmetic,
   const_fn_fn_ptr_basics,
   const_fn_trait_bound,
-  const_fn_union,
-  const_generics,
   const_intrinsic_copy,
-  const_maybe_uninit_as_ptr,
+  const_maybe_uninit_as_mut_ptr,
   const_maybe_uninit_assume_init,
   const_mut_refs,
-  const_panic,
   const_precise_live_drops,
   const_ptr_is_null,
   const_ptr_offset,
   const_ptr_offset_from,
   const_ptr_read,
   const_ptr_write,
-  const_raw_ptr_deref,
-  const_raw_ptr_to_usize_cast,
   // this should be called `const_interior_mutability` IMO
   const_refs_to_cell,
   const_replace,
@@ -58,9 +53,9 @@
   core_intrinsics,
   doc_cfg,
   exact_size_is_empty,
+  generic_const_exprs,
   inline_const,
   maybe_uninit_extra,
-  maybe_uninit_ref,
   maybe_uninit_uninit_array,
   pattern,
   slice_partition_dedup,
@@ -69,18 +64,17 @@
   trusted_random_access,
   untagged_unions
 )]
-#![cfg_attr(feature = "std", feature(read_initializer))]
+#![cfg_attr(feature = "std", feature(read_buf))]
 
 use core::cmp::{Ord, PartialEq};
 use core::intrinsics::assume;
-#[doc(hidden)]
-pub use core::iter::FromIterator;
 use core::marker::PhantomData;
 use core::mem::{self, size_of, MaybeUninit};
 use core::ops::{
   Add, Bound::Excluded, Bound::Included, Bound::Unbounded, Div, Mul, RangeBounds, Sub,
 };
 use core::ptr;
+use core::slice::{from_raw_parts, from_raw_parts_mut};
 
 pub use crate::errors::{CapacityError, PushCapacityError};
 pub use crate::heap::{
@@ -90,9 +84,7 @@ pub use crate::iterators::{
   StaticVecDrain, StaticVecIntoIter, StaticVecIterConst, StaticVecIterMut, StaticVecSplice,
 };
 pub use crate::string::{string_utils, StaticString, StringError};
-use crate::utils::{
-  const_min, quicksort_internal, reverse_copy, slice_from_raw_parts, slice_from_raw_parts_mut,
-};
+use crate::utils::{const_min, quicksort_internal, reverse_copy};
 
 #[cfg(any(feature = "std", rustdoc))]
 extern crate alloc;
@@ -118,6 +110,12 @@ pub mod utils;
 
 /// A [`Vec`](alloc::vec::Vec)-like struct (mostly directly API-compatible where it can be)
 /// implemented with const generics around an array of fixed `N` capacity.
+///
+/// Please note that while `rustdoc` does currently correctly render inherent `const fn` method
+/// signatures, the same is not true of `const` trait implementation method signatures, so at this
+/// time it's recommended that you refer directly to the source code of this crate if unsure of
+/// whether a given trait has been implemented as `const` in conjunction with the `const_trait_impl`
+/// feature.
 pub struct StaticVec<T, const N: usize> {
   // We create this field in an uninitialized state, and write to it element-wise as needed via
   // pointer methods. At no time should the regular `assume_init` function *ever* be called through
@@ -467,7 +465,13 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// ```
   #[inline(always)]
   pub const fn as_ptr(&self) -> *const T {
-    Self::first_ptr(&self.data)
+    // This is the same aliasing-avoidance / optimization-trick approach used by regular Vec.
+    let res = self.data.as_ptr() as *const T;
+    // A pointer to "element zero" of an array is never null.
+    unsafe {
+      assume(!res.is_null());
+    }
+    res
   }
 
   /// Returns a mutable pointer to the first element of the StaticVec's internal array.
@@ -485,7 +489,12 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// ```
   #[inline(always)]
   pub const fn as_mut_ptr(&mut self) -> *mut T {
-    Self::first_ptr_mut(&mut self.data)
+    // See the comment above in `as_ptr()`.
+    let res = self.data.as_mut_ptr() as *mut T;
+    unsafe {
+      assume(!res.is_null());
+    }
+    res
   }
 
   /// Returns a constant reference to a slice of the StaticVec's inhabited area.
@@ -499,7 +508,7 @@ impl<T, const N: usize> StaticVec<T, N> {
   pub const fn as_slice(&self) -> &[T] {
     // Safety: `self.as_ptr()` is a pointer to an array for which the first `length`
     // elements are guaranteed to be initialized. Therefore this is a valid slice.
-    slice_from_raw_parts(self.as_ptr(), self.length)
+    unsafe { from_raw_parts(self.as_ptr(), self.length) }
   }
 
   /// Returns a mutable reference to a slice of the StaticVec's inhabited area.
@@ -515,7 +524,7 @@ impl<T, const N: usize> StaticVec<T, N> {
   #[inline(always)]
   pub const fn as_mut_slice(&mut self) -> &mut [T] {
     // Safety: See as_slice.
-    slice_from_raw_parts_mut(self.as_mut_ptr(), self.length)
+    unsafe { from_raw_parts_mut(self.as_mut_ptr(), self.length) }
   }
 
   /// Returns a constant pointer to the element of the StaticVec at `index` without doing any
@@ -1396,7 +1405,7 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// ```
   #[inline]
   pub const fn quicksorted_unstable(&self) -> Self
-  where T: Copy + PartialOrd {
+  where T: Copy + ~const PartialOrd {
     let length = self.length;
     if length < 2 {
       // StaticVec uses specialization to have an optimized verson of `Clone` for copy types.
@@ -1431,7 +1440,7 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// ```
   #[inline]
   pub const fn quicksort_unstable(&mut self)
-  where T: Copy + PartialOrd {
+  where T: Copy + ~const PartialOrd {
     let length = self.length;
     if length < 2 {
       return;
@@ -1804,7 +1813,7 @@ impl<T, const N: usize> StaticVec<T, N> {
             .copy_to_nonoverlapping(Self::first_ptr_mut(&mut data), item_count);
           // Manually drop any excess values in the source vec to avoid undesirable memory leaks.
           if vec_len > item_count {
-            ptr::drop_in_place(slice_from_raw_parts_mut(
+            ptr::drop_in_place(from_raw_parts_mut(
               vec.as_mut_ptr().add(item_count),
               vec_len - item_count,
             ));
@@ -2115,7 +2124,7 @@ impl<T, const N: usize> StaticVec<T, N> {
       let old_length = self.length;
       unsafe {
         self.set_len(length);
-        ptr::drop_in_place(slice_from_raw_parts_mut(
+        ptr::drop_in_place(from_raw_parts_mut(
           self.mut_ptr_at_unchecked(length),
           old_length - length,
         ));
@@ -2137,7 +2146,7 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// assert_eq!(v2, [2, 3]);
   /// ```
   #[inline]
-  pub fn split_off(&mut self, at: usize) -> Self {
+  pub const fn split_off(&mut self, at: usize) -> Self {
     let old_length = self.length;
     assert!(
       at <= old_length,
@@ -2188,8 +2197,8 @@ impl<T, const N: usize> StaticVec<T, N> {
     let mut left = MaybeUninit::<[T; M]>::uninit();
     let mut right = MaybeUninit::<[T; N - M]>::uninit();
     let self_ptr = self.as_ptr();
-    let left_ptr = left.as_mut_ptr() as *mut T;
-    let right_ptr = right.as_mut_ptr() as *mut T;
+    let left_ptr = StaticVec::first_ptr_mut(&mut left);
+    let right_ptr = StaticVec::first_ptr_mut(&mut right);
     let right_length = old_length - M;
     unsafe {
       self_ptr.copy_to_nonoverlapping(left_ptr, M);
@@ -2615,7 +2624,11 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// as opposed to slices.
   #[inline(always)]
   pub(crate) const fn first_ptr(this: &MaybeUninit<[T; N]>) -> *const T {
-    this as *const MaybeUninit<[T; N]> as *const T
+    let res = this.as_ptr() as *const T;
+    unsafe {
+      assume(!res.is_null());
+    }
+    res
   }
 
   /// An internal convenience function to go from `&mut MaybeUninit<[T; N]>` to `*mut T`.
@@ -2623,7 +2636,11 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// arrays as opposed to slices.
   #[inline(always)]
   pub(crate) const fn first_ptr_mut(this: &mut MaybeUninit<[T; N]>) -> *mut T {
-    this as *mut MaybeUninit<[T; N]> as *mut T
+    let res = this.as_mut_ptr() as *mut T;
+    unsafe {
+      assume(!res.is_null());
+    }
+    res
   }
 }
 
