@@ -2,23 +2,26 @@ use core::borrow::{Borrow, BorrowMut};
 use core::cmp::{Eq, Ord, Ordering, PartialEq};
 use core::fmt::{self, Debug, Formatter};
 use core::hash::{Hash, Hasher};
-use core::iter::FromIterator;
 use core::mem::MaybeUninit;
 use core::ops::{
   Deref, DerefMut, Index, IndexMut, Range, RangeFrom, RangeFull, RangeInclusive, RangeTo,
   RangeToInclusive,
 };
 use core::ptr;
+use core::slice::{from_raw_parts, from_raw_parts_mut};
 
 use crate::heap::StaticHeap;
 use crate::iterators::{StaticVecIntoIter, StaticVecIterConst, StaticVecIterMut};
 use crate::string::StaticString;
-use crate::utils::{partial_compare, slice_from_raw_parts, slice_from_raw_parts_mut};
+use crate::utils::partial_compare;
 use crate::StaticVec;
 
 #[cfg(feature = "std")]
 use core::str;
 
+#[cfg(feature = "std")]
+use crate::utils::const_min;
+
 #[cfg(feature = "std")]
 use alloc::string::String;
 
@@ -26,12 +29,12 @@ use alloc::string::String;
 use alloc::vec::Vec;
 
 #[cfg(feature = "std")]
-use std::io::{self, BufRead, IoSlice, IoSliceMut, Read, Write};
+use std::io::{self, BufRead, IoSlice, IoSliceMut, Read, ReadBuf, Write};
 
-#[cfg(feature = "serde_support")]
+#[cfg(feature = "serde")]
 use core::marker::PhantomData;
 
-#[cfg(feature = "serde_support")]
+#[cfg(feature = "serde")]
 use serde::{
   de::{SeqAccess, Visitor},
   Deserialize, Deserializer, Serialize, Serializer,
@@ -453,8 +456,8 @@ impl<const N: usize> From<StaticString<N>> for StaticVec<u8, N> {
   }
 }
 
-/// **Note:** this is only available when the `std` crate feature is enabled.
 #[cfg(feature = "std")]
+#[doc(cfg(feature = "std"))]
 impl<T, const N: usize> From<Vec<T>> for StaticVec<T, N> {
   /// Functionally equivalent to [`from_vec`](crate::StaticVec::from_vec).
   #[inline(always)]
@@ -543,10 +546,7 @@ impl<T, const N: usize> const Index<Range<usize>> for StaticVec<T, N> {
   fn index(&self, index: Range<usize>) -> &Self::Output {
     // This is the part that confuses Clippy.
     assert!(index.start <= index.end && index.end <= self.length);
-    slice_from_raw_parts(
-      unsafe { self.ptr_at_unchecked(index.start) },
-      index.end - index.start,
-    )
+    unsafe { from_raw_parts(self.ptr_at_unchecked(index.start), index.end - index.start) }
   }
 }
 
@@ -559,10 +559,12 @@ impl<T, const N: usize> const IndexMut<Range<usize>> for StaticVec<T, N> {
   fn index_mut(&mut self, index: Range<usize>) -> &mut Self::Output {
     // This is the part that confuses Clippy.
     assert!(index.start <= index.end && index.end <= self.length);
-    slice_from_raw_parts_mut(
-      unsafe { self.mut_ptr_at_unchecked(index.start) },
-      index.end - index.start,
-    )
+    unsafe {
+      from_raw_parts_mut(
+        self.mut_ptr_at_unchecked(index.start),
+        index.end - index.start,
+      )
+    }
   }
 }
 
@@ -574,10 +576,12 @@ impl<T, const N: usize> const Index<RangeFrom<usize>> for StaticVec<T, N> {
   #[inline(always)]
   fn index(&self, index: RangeFrom<usize>) -> &Self::Output {
     assert!(index.start <= self.length);
-    slice_from_raw_parts(
-      unsafe { self.ptr_at_unchecked(index.start) },
-      self.length - index.start,
-    )
+    unsafe {
+      from_raw_parts(
+        self.ptr_at_unchecked(index.start),
+        self.length - index.start,
+      )
+    }
   }
 }
 
@@ -588,10 +592,12 @@ impl<T, const N: usize> const IndexMut<RangeFrom<usize>> for StaticVec<T, N> {
   #[inline(always)]
   fn index_mut(&mut self, index: RangeFrom<usize>) -> &mut Self::Output {
     assert!(index.start <= self.length);
-    slice_from_raw_parts_mut(
-      unsafe { self.mut_ptr_at_unchecked(index.start) },
-      self.length - index.start,
-    )
+    unsafe {
+      from_raw_parts_mut(
+        self.mut_ptr_at_unchecked(index.start),
+        self.length - index.start,
+      )
+    }
   }
 }
 
@@ -624,7 +630,7 @@ impl<T, const N: usize> const Index<RangeInclusive<usize>> for StaticVec<T, N> {
     let start = *index.start();
     let end = *index.end();
     assert!(start <= end && end < self.length);
-    slice_from_raw_parts(unsafe { self.ptr_at_unchecked(start) }, (end + 1) - start)
+    unsafe { from_raw_parts(self.ptr_at_unchecked(start), (end + 1) - start) }
   }
 }
 
@@ -637,10 +643,7 @@ impl<T, const N: usize> const IndexMut<RangeInclusive<usize>> for StaticVec<T, N
     let start = *index.start();
     let end = *index.end();
     assert!(start <= end && end < self.length);
-    slice_from_raw_parts_mut(
-      unsafe { self.mut_ptr_at_unchecked(start) },
-      (end + 1) - start,
-    )
+    unsafe { from_raw_parts_mut(self.mut_ptr_at_unchecked(start), (end + 1) - start) }
   }
 }
 
@@ -652,7 +655,7 @@ impl<T, const N: usize> const Index<RangeTo<usize>> for StaticVec<T, N> {
   #[inline(always)]
   fn index(&self, index: RangeTo<usize>) -> &Self::Output {
     assert!(index.end <= self.length);
-    slice_from_raw_parts(self.as_ptr(), index.end)
+    unsafe { from_raw_parts(self.as_ptr(), index.end) }
   }
 }
 
@@ -663,7 +666,7 @@ impl<T, const N: usize> const IndexMut<RangeTo<usize>> for StaticVec<T, N> {
   #[inline(always)]
   fn index_mut(&mut self, index: RangeTo<usize>) -> &mut Self::Output {
     assert!(index.end <= self.length);
-    slice_from_raw_parts_mut(self.as_mut_ptr(), index.end)
+    unsafe { from_raw_parts_mut(self.as_mut_ptr(), index.end) }
   }
 }
 
@@ -675,7 +678,7 @@ impl<T, const N: usize> const Index<RangeToInclusive<usize>> for StaticVec<T, N>
   #[inline(always)]
   fn index(&self, index: RangeToInclusive<usize>) -> &Self::Output {
     assert!(index.end < self.length);
-    slice_from_raw_parts(self.as_ptr(), index.end + 1)
+    unsafe { from_raw_parts(self.as_ptr(), index.end + 1) }
   }
 }
 
@@ -686,13 +689,13 @@ impl<T, const N: usize> const IndexMut<RangeToInclusive<usize>> for StaticVec<T,
   #[inline(always)]
   fn index_mut(&mut self, index: RangeToInclusive<usize>) -> &mut Self::Output {
     assert!(index.end < self.length);
-    slice_from_raw_parts_mut(self.as_mut_ptr(), index.end + 1)
+    unsafe { from_raw_parts_mut(self.as_mut_ptr(), index.end + 1) }
   }
 }
 
-/// **Note:** this is only available when the `std` crate feature is enabled.
 #[allow(clippy::from_over_into)]
 #[cfg(feature = "std")]
+#[doc(cfg(feature = "std"))]
 impl<T, const N: usize> Into<Vec<T>> for StaticVec<T, N> {
   /// Functionally equivalent to [`into_vec`](crate::StaticVec::into_vec).
   #[inline(always)]
@@ -791,15 +794,9 @@ impl_partial_ord_with_as_slice_against_slice!(&mut [T1], StaticVec<T2, N>);
 
 /// Read from a StaticVec. This implementation operates by copying bytes into the destination
 /// buffers, then shifting the remaining bytes over.
-///
-/// **Note:** this is only available when the `std` crate feature is enabled.
 #[cfg(feature = "std")]
+#[doc(cfg(feature = "std"))]
 impl<const N: usize> Read for StaticVec<u8, N> {
-  #[inline(always)]
-  unsafe fn initializer(&self) -> io::Initializer {
-    io::Initializer::nop()
-  }
-
   #[inline]
   fn read(&mut self, buf: &mut [u8]) -> io::Result<usize> {
     let current_length = self.length;
@@ -891,10 +888,20 @@ impl<const N: usize> Read for StaticVec<u8, N> {
     }
     Ok(total_read)
   }
+
+  #[inline]
+  fn read_buf(&mut self, buf: &mut ReadBuf<'_>) -> io::Result<()> {
+    // Here we directly adapt the implementation from &[u8].
+    let amount = const_min(buf.remaining(), self.len());
+    let (a, b) = self.as_mut().split_at(amount);
+    buf.append(a);
+    *self = Self::from(b);
+    Ok(())
+  }
 }
 
-/// **Note:** this is only available when the `std` crate feature is enabled.
 #[cfg(feature = "std")]
+#[doc(cfg(feature = "std"))]
 impl<const N: usize> Write for StaticVec<u8, N> {
   #[inline]
   fn write(&mut self, buf: &[u8]) -> io::Result<usize> {
@@ -934,8 +941,8 @@ impl<const N: usize> Write for StaticVec<u8, N> {
   }
 }
 
-/// **Note:** this is only available when the `std` crate feature is enabled.
 #[cfg(feature = "std")]
+#[doc(cfg(feature = "std"))]
 impl<const N: usize> BufRead for StaticVec<u8, N> {
   #[inline(always)]
   fn fill_buf(&mut self) -> io::Result<&[u8]> {
@@ -948,7 +955,8 @@ impl<const N: usize> BufRead for StaticVec<u8, N> {
   }
 }
 
-#[cfg(feature = "serde_support")]
+#[cfg(feature = "serde")]
+#[doc(cfg(feature = "serde"))]
 impl<'de, T, const N: usize> Deserialize<'de> for StaticVec<T, N>
 where T: Deserialize<'de>
 {
@@ -985,7 +993,8 @@ where T: Deserialize<'de>
   }
 }
 
-#[cfg(feature = "serde_support")]
+#[cfg(feature = "serde")]
+#[doc(cfg(feature = "serde"))]
 impl<T, const N: usize> Serialize for StaticVec<T, N>
 where T: Serialize
 {
