@@ -1,6 +1,6 @@
 use core::cmp::{Ordering, PartialOrd};
 use core::intrinsics::{assume, ptr_offset_from};
-use core::mem::{/* align_of, */ size_of, MaybeUninit};
+use core::mem::{size_of, MaybeUninit};
 
 use crate::StaticVec;
 
@@ -8,10 +8,13 @@ use crate::StaticVec;
 /// directly for possible ZSTs. This is used specifically in the iterator implementations.
 #[inline(always)]
 pub(crate) const fn distance_between<T>(dest: *const T, origin: *const T) -> usize {
+  // Safety: this function is used strictly with linear inputs where `dest` is known to come after
+  // `origin`.
   match size_of::<T>() {
-    0 => unsafe { (dest as usize).wrapping_sub(origin as usize) },
-    // Safety: this function is used strictly with linear inputs
-    // where dest is known to come after origin.
+    // If `T` is a ZST, we cannot use typed pointers as it would either return an incorrect value
+    // or in some cases segfault.
+    0 => unsafe { ptr_offset_from(dest as *const u8, origin as *const u8) as usize },
+    // For all other sizes of `T`, however typed pointers are just fine.
     _ => unsafe { ptr_offset_from(dest, origin) as usize },
   }
 }
@@ -83,14 +86,21 @@ where T: Copy {
   }
 }
 
-/// An internal convenience function for incrementing mutable ZST pointers by usize offsets.
-pub(crate) const fn zst_ptr_add_mut<T>(ptr: *mut T, offset: usize) -> *mut T {
-  unsafe { (ptr as usize + offset) as *mut T }
+/// An internal convenience function for incrementing immutable ZST pointers by usize offsets.
+#[inline(always)]
+pub(crate) const fn zst_ptr_add<T>(ptr: *const T, count: usize) -> *const T {
+  debug_assert!(size_of::<T>() == 0, "`zst_ptr_add` called on a non-ZST!");
+  (ptr as *const u8).wrapping_add(count) as *const T
 }
 
-/// An internal convenience function for incrementing immutable ZST pointers by usize offsets.
-pub(crate) const fn zst_ptr_add<T>(ptr: *const T, offset: usize) -> *const T {
-  unsafe { (ptr as usize + offset) as *const T }
+/// An internal convenience function for incrementing mutable ZST pointers by usize offsets.
+#[inline(always)]
+pub(crate) const fn zst_ptr_add_mut<T>(ptr: *mut T, count: usize) -> *mut T {
+  debug_assert!(
+    size_of::<T>() == 0,
+    "`zst_ptr_add_mut` called on a non-ZST!"
+  );
+  (ptr as *mut u8).wrapping_add(count) as *mut T
 }
 
 /// A version of the default `partial_cmp` implementation with a more flexible function signature.
@@ -116,7 +126,7 @@ pub(crate) fn partial_compare<T1, T2: PartialOrd<T1>>(
 /// A simple quicksort function for internal use, called in
 /// ['quicksorted_unstable`](crate::StaticVec::quicksorted_unstable).
 #[inline]
-pub(crate) const fn quicksort_internal<T: Copy + PartialOrd>(
+pub(crate) const fn quicksort_internal<T: Copy + ~const PartialOrd>(
   values: *mut T,
   mut low: isize,
   mut high: isize,
@@ -167,56 +177,3 @@ pub(crate) const fn quicksort_internal<T: Copy + PartialOrd>(
     }
   }
 }
-
-/*
-/// A local (identically written) `const fn` version of `intrinsics::is_aligned_and_not_null`.
-#[inline(always)]
-pub(crate) const fn is_aligned_and_not_null<T>(ptr: *const T) -> bool {
-  // This does not compile currently in certain const contexts because of the `ptr as usize` cast,
-  // even though the results of the below slice functions are always valid and properly usable
-  // from regular code even when declared as `static`.
-  unsafe { !ptr.is_null() && ptr as usize % align_of::<T>() == 0 }
-}
-*/
-
-/// A local (identically written) `const fn` version of `slice::from_raw_parts`.
-#[inline(always)]
-pub(crate) const fn slice_from_raw_parts<'a, T>(data: *const T, length: usize) -> &'a [T] {
-  debug_assert!(
-    /*
-    is_aligned_and_not_null(data),
-    "Attempted to create an unaligned or null slice!"
-    */
-    // See comment starting at line 165 for more info about what's going on here. Note that the
-    // alignment check is not actually a concern for our use case anyways, since we only call this
-    // function with known-valid pointers to initialized elements of a StaticVec's internal array.
-    !data.is_null(),
-    "Attempted to create a null slice!"
-  );
-  debug_assert!(
-    size_of::<T>().saturating_mul(length) <= isize::MAX as usize,
-    "Attempted to create a slice covering at least half of the address space!"
-  );
-  unsafe { &*core::ptr::slice_from_raw_parts(data, length) }
-}
-
-/// A local (identically written) `const fn` version of `slice::from_raw_parts_mut`.
-#[inline(always)]
-pub(crate) const fn slice_from_raw_parts_mut<'a, T>(data: *mut T, length: usize) -> &'a mut [T] {
-  debug_assert!(
-    /*
-    is_aligned_and_not_null(data),
-    "Attempted to create an unaligned or null slice!"
-    */
-    // See comment starting at line 165 for more info about what's going on here. Note that the
-    // alignment check is not actually a concern for our use case anyways, since we only call this
-    // function with known-valid pointers to initialized elements of a StaticVec's internal array.
-    !data.is_null(),
-    "Attempted to create a null slice!"
-  );
-  debug_assert!(
-    size_of::<T>().saturating_mul(length) <= isize::MAX as usize,
-    "Attempted to create a slice covering at least half of the address space!"
-  );
-  unsafe { &mut *core::ptr::slice_from_raw_parts_mut(data, length) }
-}
